import hmac
import hashlib
import base64

from .constants import (
    SMTP_REGIONS,
    SIG_DATE,
    SIG_SERVICE,
    SIG_TERMINAL,
    SIG_MESSAGE,
    SIG_VERSION,
)


def sign(key, msg):
    return hmac.new(key, msg.encode("utf-8"), hashlib.sha256).digest()


def calculate_key(secret_access_key, region):
    assert region in SMTP_REGIONS, f"The {region} Region doesn't have an SMTP endpoint."

    signature = sign(("AWS4" + secret_access_key).encode("utf-8"), SIG_DATE)
    signature = sign(signature, region)
    signature = sign(signature, SIG_SERVICE)
    signature = sign(signature, SIG_TERMINAL)
    signature = sign(signature, SIG_MESSAGE)
    signature_and_version = bytes([SIG_VERSION]) + signature
    smtp_password = base64.b64encode(signature_and_version)
    return smtp_password.decode("utf-8")



#!/usr/bin/env python3
import argparse
from getpass import getpass

from .constants import SMTP_REGIONS
from .lib import calculate_key


def run():
    parser = argparse.ArgumentParser(description="Convert a Secret Access Key for an IAM user to an SMTP password.")
    parser.add_argument("--secret", help="The Secret Access Key to convert.")
    parser.add_argument(
        "--region",
        help="The AWS Region where the SMTP password will be used.",
        choices=SMTP_REGIONS,
    )
    args = parser.parse_args()

    region = args.region if args.region else input("region: ").strip()
    secret = args.secret if args.secret else getpass("secret: ").strip()
    print(calculate_key(secret, region))


if __name__ == "__main__":
    run()



SMTP_REGIONS = [
    "us-east-2",  # US East (Ohio)
    "us-east-1",  # US East (N. Virginia)
    "us-west-2",  # US West (Oregon)
    "ap-south-1",  # Asia Pacific (Mumbai)
    "ap-northeast-2",  # Asia Pacific (Seoul)
    "ap-southeast-1",  # Asia Pacific (Singapore)
    "ap-southeast-2",  # Asia Pacific (Sydney)
    "ap-northeast-1",  # Asia Pacific (Tokyo)
    "ca-central-1",  # Canada (Central)
    "eu-central-1",  # Europe (Frankfurt)
    "eu-west-1",  # Europe (Ireland)
    "eu-west-2",  # Europe (London)
    "sa-east-1",  # South America (Sao Paulo)
    "us-gov-west-1",  # AWS GovCloud (US)
]

# These values are required to calculate the signature. Do not change them.
SIG_DATE = "11111111"
SIG_SERVICE = "ses"
SIG_MESSAGE = "SendRawEmail"
SIG_TERMINAL = "aws4_request"
SIG_VERSION = 0x04
